#!/usr/bin/env bash

INSIGHT_SERVER=/insight-server
export JFI_HOME=/opt/jfrog
export JFIS_APP_NAME=${JFIS_APP_NAME:-insight-server}
export JFI_HOME_CORE=${JFI_HOME}/${JFIS_APP_NAME}
export JFIS_LOGS=${JFIS_LOGS:-$JFI_HOME_CORE/logs}
export JFIS_SECURITY=${JFI_HOME_CORE}/_MASTER_/data/contexts/security

[ ! -z ${JFMC_MISSION_CONTROL_CERT} ] || errorExit "JFMC_MISSION_CONTROL_CERT env variable is not set, please set this with mission-control cert location"
[ ! -z ${JFMC_INSIGHT_SERVER_CERT}  ] || errorExit "JFMC_INSIGHT_SERVER_CERT env variable is not set, please set this with insight-server cert location"
[ ! -z ${JFMC_INSIGHT_SERVER_KEY}   ] || errorExit "JFMC_INSIGHT_SERVER_KEY env variable is not set, please set this with insight-server key location"

logger() {
    DATE_TIME=$(date +"%Y-%m-%d %H:%M:%S")
    if [ -z "$CONTEXT" ]
    then
        CONTEXT=$(caller)
    fi
    MESSAGE=$1
    CONTEXT_LINE=$(echo "$CONTEXT" | awk '{print $1}')
    CONTEXT_FILE=$(echo "$CONTEXT" | awk -F"/" '{print $NF}')
    printf "%s %05s %s %s\n" "$DATE_TIME" "[$CONTEXT_LINE" "$CONTEXT_FILE]" "$MESSAGE"
    CONTEXT=
}

errorExit () {
    echo; logger "ERROR: $1"; echo
    exit 1
}

warn () {
    logger "WARNING: $1"
}

# Test directory has read/write permissions for current user
testDirectoryPermissions () {
    local dir_to_check=$1
    local error=false
    local error_type=

    [ -d ${dir_to_check} ] || errorExit "'${dir_to_check}' is not a directory"

    logger "Testing directory (${dir_to_check}) has read/write permissions"

    local test_file=${dir_to_check}/test-permissions

    # Write file
    if echo test > ${test_file} 1> /dev/null 2>&1; then
        # Write succeeded. Testing read...
        if cat ${test_file} > /dev/null; then
            logger "${dir_to_check} has read/write permissions for ${JFMC_USER_NAME}"
            rm -f ${test_file}
        else
            error=true
            error_type=readable
        fi
    else
        error=true
        error_type=writable
    fi

    if [ "${error}" == true ]; then
        local stat_data=$(stat -Lc "Directory: %n, permissions: %a, owner: %U, group: %G" ${dir_to_check})
        logger "###########################################################"
        logger "${dir_to_check} is NOT ${error_type}!"
        logger "${stat_data}"
        logger "Mounted directory must be ${error_type} by user '${JFMC_USER_NAME}' (id ${JFMC_USER_ID})"
        logger "###########################################################"
        errorExit "Directory ${dir_to_check} has bad permissions for user '${JFMC_USER_NAME}'"
    fi
}

testFilePermissions(){
    local file_to_check=$1
    local error=false
    local error_type=

    [ -f ${file_to_check} ] || errorExit "unable to access '${file_to_check}'"

    logger "Testing file (${file_to_check}) has read permissions"

    # Testing read...
    if cat ${test_file} > /dev/null; then
        logger "${file_to_check} has read permissions for ${JFMC_USER_NAME}"
    else
        error=true
        error_type=readable
    fi

    if [ "${error}" == true ]; then
        local stat_data=$(stat -Lc "File: %n, permissions: %a, owner: %U, group: %G" ${file_to_check})
        logger "###########################################################"
        logger "${file_to_check} is NOT ${error_type}!"
        logger "${stat_data}"
        logger "Mounted file must be ${error_type} by user '${JFMC_USER_NAME}' (id ${JFMC_USER_ID})"
        logger "###########################################################"
        errorExit "File ${file_to_check} has bad permissions for user '${JFMC_USER_NAME}'"
    fi
}

# Test permissions on JFIS_LOGS and JFIS_SECURITY
testPermissions () {
    # JFIS_LOGS folder
    testDirectoryPermissions ${JFIS_LOGS}

    # JFIS_SECURITY files
    testFilePermissions ${JFMC_MISSION_CONTROL_CERT}
    testFilePermissions ${JFMC_INSIGHT_SERVER_CERT}
    testFilePermissions ${JFMC_INSIGHT_SERVER_KEY}
}

# In case data dirs are missing or not mounted, need to create them
setupDirs () {
    logger "Setting up directories"
    mkdir -p "${JFIS_LOGS}" || errorExit "unable to create following directories : ${JFIS_LOGS}"
}

setupDirs
testPermissions

JFMC_CONSOLE_LOG="${JFIS_LOGS}"/"${JFIS_APP_NAME}".log
# Write log to console and a file
exec "${INSIGHT_SERVER}" 2>&1 | tee  -a "${JFMC_CONSOLE_LOG}"